<?php

namespace Service;

use Service\Manager\Mongo\Types\DateType;
use Service\Manager\Mongo\Types\Type;
use Teknisa\Libs\Util\Utilities as LibUtilities;
use Util\Period;
use Util\Utilities;
use Util\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Util\InstanceProvider;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use function React\Promise\all;

class Api {

	protected $mongoManager;
	const PAGE_SIZE = 3000;
	const API_DATA_SOURCE_COLLECTION = 'apiDataSource';
	const TEMP_API_DATA_SOURCE_COLLECTION = 'tempApiDataSource';
	const API_POWER_BI_START =  "= Table.FromList(Json.Document(Web.Contents(\"";
    const API_POWER_BI_MID =  "\", [Headers=[#\"Content-Type\"=\"application/json\", WEB_TOKEN=\"";
    const API_POWER_BI_END =  "\"]])), Splitter.SplitByNothing(), null, null, ExtraValues.Error)";

	public function __construct()
	{
        $this->mongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
    }

    /**
     * @param $apiDataSource
     * @param \Zeedhi\Framework\Session\Session $session
     * @throws \Exception
     */
    public static function setSessionExternalApi($apiDataSource, \Zeedhi\Framework\Session\Session $session, $nrorgOracle)
    {
        /** @var \Teknisa\Libs\Service\Api $apiServiceLib */
        /** @var \Service\ZmartOperator $zmartOperator */
        $zmartOperator= InstanceManager::getInstance()->getService('\Service\ZmartOperator');
        $userData = $zmartOperator->getUserData($nrorgOracle,$apiDataSource['EMAIL_API']);
        $user = $userData['USER'];
        $operatorCd = $user['ID'];
        $accessProfile = array();
        if(!empty($userData['ACCESS_PROFILE'])){
            $accessProfile = LibUtilities::getArrayFromAssociativeArray($userData['ACCESS_PROFILE']);
        }

        if($user['IS_ACTIVE'] != 'Y'){
            Exception::operatorInactive();
        }
        if(empty($accessProfile)){
            Exception::accessProfileEmpty();
        }
        $session->set('USER_ID', $operatorCd);
        $session->set('EMAIL_API', $apiDataSource['EMAIL_API']);
        $session->set('NAME', $user['NAME']);
        $session->set('CDOPERADOR', $operatorCd);
        $session->set('ORGANIZATION_ID', $user['NRORG']);
        $session->set('ACCESS_PROFILE', $accessProfile);
        /** @var Environment $envoronment */
        $envoronment = InstanceManager::getInstance()->getService('environment');
        $envoronment->setUserId($operatorCd);
        $envoronment->setOrganizationId($user['NRORG']);
        $envoronment->setAclProfiles($accessProfile);
    }

    /**
     * @param $apiDataSource
     * @return array
     */
    public static function getArrayValuesExternalApi($apiDataSource): array
    {
        $start = $apiDataSource['FIELD_DATE_PERIOD']['start'];
        $end = $apiDataSource['FIELD_DATE_PERIOD']['end'];
        if (!empty($apiDataSource['PERIOD_CODE'])) {
            $expresion = $apiDataSource['PERIOD_CODE'];
            list($start, $end) = \Util\Period::decodeFormatted($expresion);
        }
        $data = array(
            'API_ID' => $apiDataSource['API_ID'],
            'PAGE' => 1,
            'FIELD_DATE_NAME' => $apiDataSource['FIELD_DATE_NAME'],
            'FIELD_DATE_START' => $start,
            'FIELD_DATE_END' => $end
        );
        return $data;
    }

    /**
     * @param $nrorg
     * @param $token
     * @return array|mixed
     * @throws \Exception
     */
    public static function findWebTokenExternalApi($nrorg, $token)
    {
        /** @var Environment $envoronment */
        $envoronment = InstanceManager::getInstance()->getService('environment');
        $envoronment->setOrganizationId($nrorg);
        /** @var \Service\Manager\Mongo\Manager $organizationMongoManager */
        $organizationMongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        $apiDataSource = self::findByWebToken($organizationMongoManager, self::API_DATA_SOURCE_COLLECTION, $token);
        if (empty($apiDataSource)) {
            \Service\Exception::webTokenInvalid();
        }
        return $apiDataSource[0];
    }

    public static function findWebTokenByTempToken($nrorg, $token)
    {
        /** @var Environment $envoronment */
        $envoronment = InstanceManager::getInstance()->getService('environment');
        $envoronment->setOrganizationId($nrorg);
        /** @var \Service\Manager\Mongo\Manager $organizationMongoManager */
        $organizationMongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        $tempApiDataSource = self::findByTempToken($organizationMongoManager, self::TEMP_API_DATA_SOURCE_COLLECTION, $token);
        if (empty($tempApiDataSource->getRows())) {
            \Service\Exception::tempWebTokenInvalid();
        }
        $token = $tempApiDataSource->getRows()[0]["WEB_TOKEN"];
        $organizationMongoManager->delete($tempApiDataSource);
        return $token;
    }

    /**
     * @param $nrorg
     * @return \Zeedhi\Framework\Session\Session
     */
    public static function setOrganizationApi($nrorg): \Zeedhi\Framework\Session\Session
    {
        $session = \Teknisa\Libs\Util\InstanceProvider::getSession();
        $session->set('NRORG', $nrorg);
        $session->set('organizationId', $nrorg);
        return $session;
    }

    /**
     * @param $webToken
     * @return array
     * @throws \Exception
     */
    public static function getTokenAndNrorgByWebToken($webToken): array
    {
        $webToken = base64_decode($webToken);
        list($token, $nrorg) = explode('_', $webToken);
        if (empty($token) || empty($nrorg)) {
            \Service\Exception::webTokenInvalid();
        }
        return array($token, $nrorg);
    }

    public function findBiggerStructure($zmartStructureFilterId)
	{
		$filterData = new FilterCriteria('structure');
		$filterData->addCondition('externalIdString', $zmartStructureFilterId);
        $dataSet = $this->mongoManager->findBy($filterData);
        return $dataSet->getRows();
    }

	public function findbuildWidgetByViewId($environment, $viewId, $drillDownFilters)
	{
		$this->setSessionVariables($environment);
		$structureId = InstanceManager::getInstance()->getService('\Service\Widget')->getUserHierarchysTrees();
		return $structureId;
	}

	public function setSessionVariables($environment)
	{
		$operatorCd = $environment['OPERATOR_CD'];
		$accessProfile = $environment['ACCESS_PROFILE'];
		$session = InstanceManager::getInstance()->getService('session');
		$session->set('USER_ID', $operatorCd);
		$session->set('CDOPERADOR', $operatorCd);
		$session->set('ORGANIZATION_ID', '20');
		$session->set('ACCESS_PROFILE', $accessProfile);
		$envoronment = InstanceManager::getInstance()->getService('environment');
		$envoronment->setUserId($operatorCd);
		$envoronment->setOrganizationId('20');
		$envoronment->setAclProfiles($accessProfile);
	}

    public function findDataSet($apiId,$filter,$page){
        list($filterCriteria, $organizationMongoManager) = $this->getFilterCriteriaFindDataSet($apiId, $filter, $page);

        $dataSet = $organizationMongoManager->simpleFindPagination($filterCriteria);
       // var_dump(memory_get_usage());die;
        header('Accept-Ranges: none');
        header('Content-type: application/json; charset=utf-8');
        foreach ($dataSet as $key => &$row) {
            foreach ($row as &$data) {
                if ($data instanceof \MongoDB\BSON\UTCDateTime){
                    $data = $data->toDateTime()->format('d/m/Y');
                }else if($data instanceof \MongoDB\BSON\ObjectId){
                    $data = (string)$data;
                }
            }
            $rows[$key] = $row;
        }
        return $dataSet;
    }

    public static function sizeStorageVar($var){
        $serializedFoo = serialize($var);
        if (function_exists('mb_strlen')) {
            $size = mb_strlen($serializedFoo, '8bit');
        } else {
            $size = strlen($serializedFoo);
        }
        return $size;
    }

    public function findBigDataSet($apiId,$filter,$page,$token,$download = false){
        $start = microtime(true);
        list($filterCriteria, $organizationMongoManager) = $this->getFilterCriteriaFindDataSet($apiId, $filter, $page);

        /** @var \Service\Manager\Mongo\Manager $organizationMongoManager */
        /** @var ZmartFilterCriteria $filterCriteria */
        $now =  new \DateTime();

        $now = \Zeedhi\Framework\DataSource\Manager\Mongo\Types\Type::getType('mongoDate')->convertToDatabaseValue($now->format('Y-m-d H:i:s'));
        $logUsage = array(
            'CREATED_AT'=> $now,
            'SIZE_STORAGE_VAR_PHP_IN_BYTE'=> 0,
            'SIZE_STORAGE_JSON_IN_BYTE'=> 0,
            'COUNT'=> 0,
        );

        $session = \Teknisa\Libs\Util\InstanceProvider::getSession();
        $logUsage['FILTER_DATE'] = $filter['FIELD_DATE'];
        $filterFinal =$filterCriteria->getConditions();
        if(!empty($filterFinal[1])){
            $logUsage['FILTER_STRUCTURE'] = $filterFinal[1];
        }
        $logUsage['DATA_SOURCE'] = $apiId;
        $logUsage['USER_ID'] = $session->get('USER_ID');
        $logUsage['EMAIL_API'] = $session->get('EMAIL_API');
        $logUsage['NAME'] = $session->get('NAME');
        $logUsage['WEB_TOKEN'] = $token;
        $logUsage['REMOTE_ADDR'] = empty($_SERVER['REMOTE_ADDR'])?'Not found':$_SERVER['REMOTE_ADDR'];
        $logUsage['HTTP_USER_AGENT'] = empty($_SERVER['HTTP_USER_AGENT'])?'Not found':$_SERVER['HTTP_USER_AGENT'];
        header('Accept-Ranges: none');
        if($download){
            header("Content-Disposition: attachment; filename=report.json");
        }
        echo('[');
        do{
            $filterCriteria->setPage($page);
            $dataSet = $organizationMongoManager->simpleFindPagination($filterCriteria);
            foreach ($dataSet as $key => &$row) {
               unset($row['ZM_ROWNUM']);
               foreach ($row as &$data) {
                   if ($data instanceof \MongoDB\BSON\UTCDateTime){
                       $data = $data->toDateTime()->format('d/m/Y');
                   }else if($data instanceof \MongoDB\BSON\ObjectId){
                       $data = (string)$data;
                   }
               }
               $dataSet[$key] = $row;
            }
            $jsonDataEnd='';
            if($page!=1 && !empty($dataSet) ){
                $jsonDataEnd.=',';
            }
            $page++;
            $jsonData = json_encode($dataSet,JSON_UNESCAPED_UNICODE);
            $jsonDataEnd .= substr($jsonData, 1,-1);
            echo($jsonDataEnd);
            $logUsage['SIZE_STORAGE_VAR_PHP_IN_BYTE']+= self::sizeStorageVar($dataSet);
            $logUsage['SIZE_STORAGE_JSON_IN_BYTE']+= self::sizeStorageVar($jsonDataEnd);
            $logUsage['COUNT']+= count($dataSet);
        } while(count($dataSet) === self::PAGE_SIZE);
        echo(']');

        $logUsage['TIME_USAGE_IN_SEG'] = microtime(true) - $start;
        $logUsage['MEMORY_USAGE_IN_BYTE'] = memory_get_usage();

        $organizationMongoManager->persist(new DataSet('apiUsageLog', array($logUsage)));
    }

    /**
     * @param $apiId
     * @param $filter
     * @param $page
     * @return array
     * @throws \Exception
     */
    public function getFilterCriteriaFindDataSet($apiId, $filter, $page): array
    {
        /** @var \Util\Environment $envoronment */
        $envoronment = InstanceManager::getInstance()->getService('environment');
        $envoronment->setOrganizationId($envoronment->getOrganizationId());
        $envoronment->setAclProfiles($envoronment->getAclProfilesLogin());
        $filterdate = $filter['FIELD_DATE'];
        $interval = Period::convertStringToMoment($filterdate['START'], $filterdate['END']);
        $interval[0] = Type::getType('date')->convertToDatabaseValue($interval[0]);
        $interval[1] = Type::getType('date')->convertToDatabaseValue($interval[1]);

        $filterCriteria = new ZmartFilterCriteria($apiId, array(array(
            "columnName" => $filterdate['NAME'],
            "value" => $interval,
            "operator" => FilterCriteria::BETWEEN
        )));
        $filterCriteria->setPageSize(self::PAGE_SIZE);
        $filterCriteria->setPage($page);
        $filterCriteria->addOrderBy('_id');

        /** @var \Service\Manager\Mongo\Manager $organizationMongoManager */
        $organizationMongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        $organizationMongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        /** @var Hierarchy $serviceHierarchy */
        $serviceHierarchy = InstanceManager::getInstance()->getService('\Service\Hierarchy');
        /** @var WidgetData $serviceWidgetData */
        $serviceWidgetData = InstanceManager::getInstance()->getService('\Service\WidgetData');
        /** @var DataSource $serviceDataSource */
        $serviceDataSource = InstanceManager::getInstance()->getService('\Service\DataSource');
        $structures = $serviceHierarchy->getDataSourceHierarchyByLoggedUser($apiId);
        if (empty($structures)) {
            Exception::structureNotFoundINDb();
        }
        $dataSource = $serviceDataSource->findById($apiId);
        if (empty($dataSource[0])) {
            Exception::dataSourceNotFoundINDb();
        } else {
            $dataSource = $dataSource[0];
        }
        $serviceWidgetData->addWhereFromRelations($filterCriteria, $dataSource, $structures);
        return array($filterCriteria, $organizationMongoManager);
    }

    public function findApiDataSource($params){
        //  $filterCriteria = new ZmartFilterCriteria('apiDataSource', $filterCriteria->getConditions());
        //  $filterCriteria->addOrderBy('_id',FilterCriteria::ORDER_DESC);
        /** @var \Service\Manager\Mongo\Manager $organizationMongoManager */
        $organizationMongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        $rows = $organizationMongoManager->findNext('apiDataSource', $params);
        foreach ($rows as &$row) {
            $id = $row['_id'];
            $token = self::createTokenApi($id);;
            $row['API_LINK'] =  Utilities::getUrlAPiFindBigData();
            $row['WEB_TOKEN'] = $token;
            $row['API_POWER_BI'] =  self::getStringApiPowerBi($token);
            if(!empty($row['PERIOD_CODE'])){
                list($row['FIELD_DATE_PERIOD']['start'], $row['FIELD_DATE_PERIOD']['end']) = \Util\Period::decodeFormatted($row['PERIOD_CODE']);
            }
            if(!empty($row['WEB_TOKEN'])){
                $row['API_POWER_BI'] = $this->getStringApiPowerBi($row['WEB_TOKEN']);
            }
        }
        return $rows;
    }
    public static function getConditionsByTempToken($tempToken){
        list($token, $nrorg) = self::getTokenAndNrorgByWebToken($tempToken);
        self::setOrganizationApi($nrorg);
        return self::findWebTokenByTempToken($nrorg, $token);
    }

    /**
     * @param $nrorg
     * @throws \Exception
     */
    public static function setOrgDBMongo($nrorg)
    {
        /** @var Environment $envoronment */
        $envoronment = InstanceManager::getInstance()->getService('environment');
        $envoronment->setOrganizationId($nrorg);
        $envoronment->setNrorgoper($nrorg);
    }

    public static function getConditionsByWebToken($webToken){
        list($token, $nrorg) = self::getTokenAndNrorgByWebToken($webToken);
        self::setOrgDBMongo($nrorg);

        $apiDataSource = self::findWebTokenExternalApi($nrorg, $token);

        $nrorgOracle = $nrorg;
        if(!empty($apiDataSource["NRORG_ORACLE"])) {
            $nrorgOracle = $apiDataSource["NRORG_ORACLE"];
        }
        $session = self::setOrganizationApi($nrorgOracle);
        self::setSessionExternalApi($apiDataSource, $session, $nrorgOracle);
        return self::getArrayValuesExternalApi($apiDataSource);
    }

    public static function createTokenApi($idApi){
        $nrorg = Utilities::getOrganizationIdForMongo();
        return base64_encode($idApi.'_'.$nrorg);
    }

    public function saveTokenTemp($row){
        /** @var \Service\Manager\Mongo\Manager $organizationMongoManager */
        $organizationMongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');

        if(empty($row['NRORG'])){
            $row['NRORG'] = Utilities::getOrganizationIdForMongo();
        }
        $return =  $organizationMongoManager->persist(new DataSet(self::TEMP_API_DATA_SOURCE_COLLECTION, array($row)));
        $id = $return[0]['_id'];
        $token = self::createTokenApi($id);
        return array($token);
    }

    public function saveApiDataSource($row){
        /** @var \Service\Manager\Mongo\Manager $organizationMongoManager */
        $organizationMongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        $dataAfter = null;
        if(!empty($row['_id'])){
            $dataAfter = $this->findById($organizationMongoManager, self::API_DATA_SOURCE_COLLECTION,  $row['_id']);
        }

        if(empty($row['EMAIL_API'])){
            $row['EMAIL_API'] = InstanceProvider::getSession()->get('EMAIL');
        }

        if(empty($row['NRORG'])){
            $row['NRORG'] = Utilities::getOrganizationIdForMongo();
        }
        $row['NRORG_ORACLE'] = Utilities::getOrganizationIdForFindAcl();
        $return =  $organizationMongoManager->persist(new DataSet('apiDataSource', array($row)));
        $id = $return[0]['_id'];
        $token = self::createTokenApi($id);;

        //list($row['API_LINK'],$row['WEB_TOKEN'],$row['API_POWER_BI']) = $this->getApiLink($row['EMAIL_API'],$id,$row['NRORG']);
        $return[0]['API_LINK'] =  Utilities::getUrlAPiFindBigData();
        $return[0]['WEB_TOKEN'] = $token;
        $return[0]['API_POWER_BI'] =  self::getStringApiPowerBi($token);
        $return[0]['EMAIL_API'] = $row['EMAIL_API'];
        $return[0]['NRORG'] = $row['NRORG'];
        $this->setLog($organizationMongoManager,self::API_DATA_SOURCE_COLLECTION,$id,$dataAfter);
        return $return;
    }

    public function deleteApiDataSource($apiDataSources) {
        $dataSet = new DataSet(self::API_DATA_SOURCE_COLLECTION, $apiDataSources);

        /** @var \Service\Manager\Mongo\Manager $organizationMongoManager */
        $organizationMongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        $dataBefores = array();
        foreach ($apiDataSources as $row) {
            if(!empty($row['_id'])){
                $dataBefores[] = $this->findById($organizationMongoManager, self::API_DATA_SOURCE_COLLECTION, $row['_id']);
            }
        }
        $return =  $organizationMongoManager->delete($dataSet);
        foreach ($dataBefores as $dataBefore) {
            if(!empty($dataBefore[0]['API_ID'])){
                $collectionId = $dataBefore[0]['API_ID'];
                 Utilities::setLog($dataBefore, array(), self::API_DATA_SOURCE_COLLECTION,$collectionId);
            }
        }

        return $return;
    }

    public static function findByWebToken(\Service\Manager\Mongo\Manager $organizationMongoManager,$dataSourceName,$id){
        $filterCriteria = new FilterCriteria( $dataSourceName, array(array(
            "columnName"=>"_id",
            "value" => $id,
            "operator" => "="
        )));

        return $organizationMongoManager->findBy($filterCriteria)->getRows();
    }

    public static function findByTempToken(\Service\Manager\Mongo\Manager $organizationMongoManager,$dataSourceName,$id){
        $filterCriteria = new FilterCriteria( $dataSourceName, array(array(
            "columnName"=>"_id",
            "value" => $id,
            "operator" => "="
        )));

        return $organizationMongoManager->findBy($filterCriteria);
    }

    public function findById(\Service\Manager\Mongo\Manager $organizationMongoManager,$dataSourceName,$id){
        $filterCriteria = new FilterCriteria( $dataSourceName, array(array(
            "columnName"=>"_id",
            "value" => $id,
            "operator" => "="
        )));

        return $organizationMongoManager->findBy($filterCriteria)->getRows();
    }

    public function setLog(\Service\Manager\Mongo\Manager $organizationMongoManager,$dataSourceName,  $id, $dataBefore)
    {
        $dataAfter = $this->findById($organizationMongoManager, $dataSourceName,  $id);
        $collectionId = $dataAfter[0]['API_ID'];
        Utilities::setLog($dataBefore, $dataAfter, $dataSourceName, $collectionId);
    }

    /**
     * @param $baseUrlApi
     * @param $token
     * @return string
     */
    public static function getStringApiPowerBi($token): string
    {
        $baseUrlApi = Utilities::getUrlAPiFindBigData();
        return self::API_POWER_BI_START . $baseUrlApi . self::API_POWER_BI_MID . $token . self::API_POWER_BI_END;
    }
}
