/*
NOME: 0007_CARSTPO_USR_EXPORTAESTOQUE.SQL
DESCRIO: Procedimento para exportao da posio de estoque e ajustes de estoque.
CRIAO: 30/01/2012 - Marcelo Alexsandro Souza Silva
*/

CREATE OR REPLACE PROCEDURE USR_EXPORTAESTOQUE(
   P_DTINVENT IN LANCTOESTOQ.DTLANCESTQ%TYPE)
AS
   V_DTGRV  DATE;
   V_HRGRV  VARCHAR2(04);
   V_RETURN VARCHAR2(1);
   V_ERROR VARCHAR2(4000);
BEGIN
   V_RETURN := USR_FUNC_BLOQUEIO('USR_POSICAOESTOQUE', 'B');
   IF V_RETURN = 'S' THEN
      V_DTGRV := NULL;
      V_HRGRV := NULL;

      /* Retorna data e hora de exportao */
      BEGIN
        SELECT TO_DATE(SYSDATE), TO_CHAR(SYSDATE,'HH24MI')
        INTO V_DTGRV, V_HRGRV
        FROM DUAL;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           V_DTGRV := NULL;
           V_HRGRV := NULL;
      END;

      /* Exclui toda posio de estoque da filial para a data informada */
      DELETE FROM USR_POSICAOESTOQUE
      WHERE DATAPOSICAO = TO_DATE(P_DTINVENT)
       AND IDORIGREGISTRO = 'T';

      /* Verificao dos lanamentos de estoque referente a posio de estoque */
      FOR C_MOVESTO IN (SELECT B.CDFILIAL, P.CDARVPROD CDPRODUTO, TO_DATE(P_DTINVENT) AS DTPOSIESTQ,
                              B.QTESTOQDIA, CUSTO_PRODUTO(B.CDFILIAL, B.CDPRODUTO, 3, B.DTPOSIESTQ -1, B.DTPOSIESTQ -1) AS VRESTOQDIA,
                              P.NMPRODUTO
                         FROM POSESTDIA B,
                              PRODUTO P,
                              (SELECT CDFILIAL
                                 FROM FILGRUPFILI
                                WHERE FILGRUPFILI.CDGRUPFILI = '9999999999') F
                        WHERE B.CDFILIAL   = F.CDFILIAL
                          AND B.CDPRODUTO  = P.CDPRODUTO
                          AND B.QTESTOQDIA <> 0
                          AND B.DTPOSIESTQ = (SELECT MAX(P.DTPOSIESTQ)
                                                 FROM POSESTDIA P
                                                WHERE P.CDFILIAL     = B.CDFILIAL
                                                  AND P.CDPRODUTO    = B.CDPRODUTO
                                                  AND P.DTPOSIESTQ   <= P_DTINVENT))
      LOOP
        /* Insere dados referente a posio de estoque */
        INSERT INTO USR_POSICAOESTOQUE
           (CDFILIAL,CDPRODUTO,DATAPOSICAO,
            QTDEPOSICAO,VRCUSTPOSICAO,IDORIGREGISTRO,
            DTGRAVACAO,HRGRAVACAO,DTIMPORTACAO,HRIMPORTACAO, NMPRODUTO)
        VALUES
           (C_MOVESTO.CDFILIAL, C_MOVESTO.CDPRODUTO, C_MOVESTO.DTPOSIESTQ,
            C_MOVESTO.QTESTOQDIA, C_MOVESTO.VRESTOQDIA, 'T',
            V_DTGRV, V_HRGRV, NULL, NULL, C_MOVESTO.NMPRODUTO);
      END LOOP;
      COMMIT;
      V_RETURN := USR_FUNC_BLOQUEIO('USR_POSICAOESTOQUE', 'D');
   END IF;
EXCEPTION
  WHEN OTHERS THEN BEGIN
     IF V_RETURN = 'S' THEN
        V_ERROR:= SQLERRM(SQLCODE);
        V_RETURN := USR_FUNC_BLOQUEIO('USR_POSICAOESTOQUE', 'D', V_ERROR);
     END IF;
  END;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('USR_EXPORTAESTOQUE','5.09.xxx'); END;
/
