import Messages from '@/Services/Messages';
import { TextInput, DialogService, LoadingService } from "@zeedhi/common";
import { Metadata, Http, DateHelper } from '@zeedhi/core';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';
import { PeriodsService } from "./PeriodsService";

export class Utilities {

    public static getNameDynamic(name: string) {
        const date = new Date();
        return name + date.getTime();
    }

    public static lightColorHexadecimalValidity(color: String) {
        const hex = color.replace('#', '');

        const r = parseInt(hex.substr(0, 2), 2);
        const g = parseInt(hex.substr(2, 2), 2);
        const b = parseInt(hex.substr(4, 2), 2);

        const lum = (r * 299 + g * 587 + b * 114) / 1000;
    }

    public static roundFloat(num: number, decimalPlaces: number = 2) {
        const tenPower = 10 ** decimalPlaces;
        return Math.round((num + Number.EPSILON) * tenPower) / tenPower;
    }

    public static fixBugHintOnfocus(component: TextInput, element: HTMLElement | undefined, count = 1) {
        setTimeout(() => {
            if (element) {
                const messages = element.querySelector('.v-messages__message');
                if (messages) {
                    if (component.hint) {
                        messages.innerHTML = component.hint;
                    }

                } else if (count < 5) {
                    count += 1;
                    Utilities.fixBugHintOnfocus(component, element, count);
                }
            }

        });
    }

    public static async genericBeforeSaveTekGrid(params: any, route: string, tekGridName: string) {
        params.event.preventDefault();
        try {
            LoadingService.show();
            await Http.post(route, {
                "requestType": "DataSet",
                "dataset": params.component.getEditedRows()
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            params.component.cancelEditedRows();
            Metadata.getInstance<TekGrid>(tekGridName).datasource.get();
        } catch (e) {
            LoadingService.hide();
            CustomAlertService.errorAlert(e);
        }
    }

    // Before Delete Methods

    public static deleteRoute: string;
    public static deleteTekGridName: string;
    public static deleteRequestType: string;

    public static genericBeforeDeleteTekGrid(params: any, route: string, tekGridName: string, requestType: string = 'DataSet') {
        params.event.preventDefault();
        this.deleteRoute = route;
        this.deleteTekGridName = tekGridName;
        this.deleteRequestType = requestType;
        DialogService.show({
          name: 'delete-dialog_' + tekGridName,
          type: 'warning',
          title: ' ',
          text: 'Deseja remover o(s) registro(s) selecionado(s)?',
          buttons: [
            { name: 'não', component: 'ZdButton', label: 'NÃO', outline: true, events: { click: () => DialogService.hide(), }, },
            { name: 'sim', component: 'ZdButton', label: 'SIM', events: Utilities.deleteTekGrid, },
          ],
        });
    }

    public static concatObjectsByUniqueId(obj1: any[], obj2: any[]): any[] {
        const obj2Values = Object.values(obj2);
        obj2Values.forEach((obj2Item) => {
            const foundItem = obj1.find((obj1Item) => obj1Item._id === obj2Item._id);
            if (!foundItem) {
                obj1.push(obj2Item);
            }
        });
        return obj1;
    }

    private static deleteTekGrid: any = {
        click: () => Utilities.doDeleteTekGrid(),
    };

    public static messageReplace(message: string, params: any) {
        Object.keys(params).forEach((key: any) => {
            const placeholder = "__" + key + "__";
            while (message.includes(placeholder)) {
                message = message.replace(placeholder, params[key]);
            }
        });
        return message;
    }

    public static saveCanvasToPng(imageData: any, filename: any) {
        try {
            const xhr = new XMLHttpRequest();
            xhr.responseType = 'blob';
            xhr.onload = function () {
                const a = document.createElement('a');
                a.href = window.URL.createObjectURL(xhr.response);
                a.download = filename + ".png";
                a.style.display = 'none';
                document.body.appendChild(a);
                a.click();
                a.remove();
            };
            xhr.open('GET', imageData);
            xhr.send();
        } catch (err) {
            let message = "Ocorreu um erro ao gerar a imagem do indicador";
            if (err instanceof Error) {
                message += ": " + err.message;
            }
            CustomAlertService.errorAlert(message);
        }
    }

    public static widgetErrorMessageAlerts(errors: any) {
        errors.forEach((err: any) => CustomAlertService.errorAlert(err));
    }

    private static async doDeleteTekGrid() {
        DialogService.hide();
        try {
            const grid = Metadata.getInstance<TekGrid>(this.deleteTekGridName);
            const requestTypeAttr = this.deleteRequestType === "DataSet" ? "dataset" : "row";
            LoadingService.show();
            await Http.post(Utilities.deleteRoute, {
                "requestType": this.deleteRequestType,
                [requestTypeAttr]: grid.selectedRows
            });
            LoadingService.hide();
            CustomAlertService.deleteSuccessAlert();
            grid.datasource.get();
            grid.selectedRows = [];
        } catch (e) {
            LoadingService.hide();
            try {
                if (e.response && e.response.data && e.response.data && e.response.data.error) {
                    const exception: any = JSON.parse(e.response.data.error.replace("Exception: ", ""));
                    if (exception && exception.message && ["CANNOT_DELETE_DATASOURCE", "CANNOT_DELETE_WIDGET", "CANNOT_DELETE_INDICATOR"].includes(exception.message)) {
                        CustomAlertService.errorAlert(Utilities.messageReplace(Messages[exception.message as keyof typeof Messages], exception.params));
                    }
                } else {
                    CustomAlertService.errorAlert(e);
                }
            } catch (er) {
                CustomAlertService.errorAlert(er);
            }
        }
        this.deleteRoute = '';
        this.deleteTekGridName = '';
    }

    public static async getLoggedUser() {
        const res = await Http.post("/getLoggedUser", {nrorg: null, nrorgPrincipal: null});
        return res.data;
    }

    public static async generatePeriodHelperOptions(period: any, helperValue: any) {
        this.unregisterPeriod(period);
        period.helperOptions = await this.getHelperOptions();
        period.helperValue = helperValue;

        if (typeof helperValue === 'string') {
            period.updateHelperHint(helperValue);
        }
    }

    public static unregisterPeriod(period: any) {
        if (period.helperOptions && period.helperOptions.length > 0) {
            period.helperOptions.forEach((option: any) => DateHelper.unregister(option.periodsCode || option._id));
        }
    }

    public static indicatorCanEnableClickEvent(indicator: any) {
        return (["drillDown", "drillDownPre", "drillDownPos"].includes(indicator.actionOnClick) && indicator.hasDrillDown) || ["filter", "filterWithPeriod", "filterPre", "filterPos"].includes(indicator.actionOnClick);
    }

    public static async getHelperOptions() {
        const res = await Http.post("/periods");
        const data = res.data.data;
        data.forEach((option: any) => DateHelper.register(option.periodsCode || option._id, option.name, () => {
            let arrayDate = [];
            if (option.periodsCode) {
                const service = new PeriodsService();
                const periods = service.decodePeriod(option.periodsCode);
                arrayDate = [periods.start, periods.end].map((date: any) => date.split('/').reverse().join('-')) as any;
            } else {
                arrayDate = [option.interval.start, option.interval.end].map((date: any) => date.split('/').reverse().join('-')) as any;
            }
            return arrayDate;
        }));
        return data.map((option: any) => option.periodsCode || option._id).filter((val: any) => val);
    }

    public static getDateFormatField(value: any, splitter: string) {
        let period = "";
        if (value) {
            if (typeof value === 'string') {
                const values = DateHelper.getValue(value, 'DD/MM/YYYY');
                period = values[0] + splitter + values[1];
            } else if (value.start && value.end) {
                period = value.start + splitter + value.end;
            } else if (Array.isArray(value)) {
                period = value[0] + splitter + value[1];
            } else {
                period = value;
            }
        }
        return period;
    }

    public static setPeriodHelperValue(period: any, value: any) {
        if (value) {
            const periodFormat = this.getDateFormatField(value, period.splitter);
            period.setDateValue(periodFormat);
            if (typeof value === 'string') {
                if (period.helperOptions?.includes(value)) {
                    period.updateHelperHint(value);
                }
            }
        } else {
            period.setDateValue("");
        }
    }

    public static async timeout(time: number) {
        return new Promise((resolve) => setTimeout(resolve, time));
    }
}
