import {Metadata, Http, RestDatasource, Loader} from '@zeedhi/core';
import {Modal, ModalService, Container, Row, SelectMultiple, Report, AlertService, Grid} from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';

export class RelatedStructuresController {
    //  Metodos de Modal
    private detailModal!: Modal;
    public async createDetailModal() {
        this.detailModal = await ModalService.createFromJson('relatedStructures/DetailModal/DetailModal', true);
    }
    public destroyDetailModal() {
        this.detailModal?.destroy();
    }
    public closeDetailModal() {
        this.detailModal.hide();
    }
    //  Metodos do Form
    protected modalData: any;
    public get currentRowId() {
        return this.modalData._id;
    }

    public async updateLoggedUserInfo() {
        const appController = Loader.getInstance('AppController');
        await appController.loginPromise.then();
        const grid = Metadata.getInstance<Grid>('structures-tek-grid');
        grid.reload();
    }

    public get listData() {
        const listEl = this.detailModal.getChildInstance<Container>("detailList");
        const lines = listEl.children.map(({name}: any) => this.getListItemValue(name, listEl.getChildInstance<Row>(name))).reduce((arr: any, el: any) => {
            el.dataSource?.forEach((datasource: any) => arr.push({...el, dataSource: datasource}));
            return arr;
        }, []).map(({structure, dataSource}: any) => ({structureColumn: structure, dataSourceColumn: dataSource}));
        return lines;
    }
    protected relationsData: any;
    public async getRelations(id: string) {
        const response = await Http.get("defaultRelations", {
            "params": {
                "in": ["structureType"],
                "structureType": id
            }
        });
        const responseData = response.data.data;
        this.relationsData = responseData.defaultRelations;
        await this.handleRelationsResponse(responseData);
    }
    public generateListItem({structure, dataSource}: any) {
        return {
            "name": structure,
            "component": "ZdRow",
            "dense": false,
            "cssClass": "zd-justify-space-between zd-align-center zd-py-4",
            "children": [
                {
                    "name": structure + "_name",
                    "component": "ZdText",
                    "text": structure,
                    "readonly": true,
                    "showLabel": false,
                    "showHelper": false,
                    "cssClass": "zd-col-4",
                    "disabled": true
                },
                {
                    "name": structure + "_select",
                    "component": "ZdSelectMultiple",
                    "value": dataSource,
                    "showHelper": false,
                    "showLabel": false,
                    "cssClass": "zd-col-7",
                    "dataText": ["label"],
                    "dataValue": "name",
                    "datasource": {
                        "route": "defaultColumns",
                        "type": "rest",
                        "lazyLoad": true,
                        "uniqueKey": "_id",
                        "loadAll": true,
                        "events": {
                            "onChangeCurrentRow": "{{RelatedStructuresController.handleResponse}}"
                        }
                    }
                }
            ]
        };
    }
    public getListItemValue(name: string, listItem: Row) {
        const relations = listItem.getChildInstance<SelectMultiple>(name + "_select");
        return {structure: name, dataSource: relations.value};
    }
    public async openDetailModal({ component, row }: any) {
        this.destroyDetailModal();
        await this.createDetailModal();
        this.modalData = row === undefined ? {} : row;
        this.detailModal.show();
    }
    public async modalMounted(params: any) {
        await this.getRelations(this.currentRowId);
    }
    public saveDetailModal() {
        const relations = this.listData;
        const content = this.relationsData[0];
        content.relations = relations;
        Http.post("defaultRelations/save", {
            "requestType": "Row",
            "row": content
        }).then(() => {
            CustomAlertService.saveSuccessAlert();
        }).catch(() => {
            CustomAlertService.errorAlert("Ocorreu um erro.");
        }).finally(() => {
            this.reloadGrid();
            this.closeDetailModal();
        });
    }
    //  Metodos do Grid
    public gridName = "structures-tek-grid";
    public async getReport({ component }: any) {
        const grid = Metadata.getInstance<TekGrid>(this.gridName);
        const report = new Report(grid, grid.title);
        const datasourceClone = new RestDatasource(grid.datasource);
        const type = component.name;
        datasourceClone.loadAll = true;
        const data = await datasourceClone.get();
        report.data = data.connection;
        if (type === "csv") {
            const workingCSVEndPoint = "https://zhreports.teknisa.cloud";
            report.endPoint = workingCSVEndPoint;
            report.fileEndPoint = workingCSVEndPoint + "/reports/";
        }
        const reportUrl = await report.getReport(type);
        window.open(reportUrl);
    }
    public async reloadGrid() {
        const grid = Metadata.getInstance<TekGrid>(this.gridName);
        return grid.reload();
    }
    public handleResponse({ component }: any) {
        const info = component.currentRow;
        if (info?.pagination) {
            if (info.defaultRelations !== undefined) {
                this.handleRelationsResponse(info);
            }
        }
    }
    public handleRelationsResponse({defaultRelations, pagination}: any) {
        const listEl = this.detailModal.getChildInstance<Container>("detailList");
        const relationsDict: any = {};
        defaultRelations.forEach((el: any) => {
            el.relations?.forEach(({structureColumn, dataSourceColumn}: any) => {
                if (structureColumn in relationsDict) {
                    relationsDict[structureColumn].push(dataSourceColumn);
                } else {
                    relationsDict[structureColumn] = [dataSourceColumn];
                }
            });
        });
        const relations = this.modalData.fieldKey.map((key: string) => ({structure: key, dataSource: relationsDict[key] ? relationsDict[key] : []}));
        listEl.children = relations.map(this.generateListItem);
    }
}
